/* File: DBOX.C */

#include "work.h"
#include "const.h"
#include "DBOX.h"
#include "EDIT.h"
#include "BUTTON.h"
#include "LBOX.h"
#include "SCROLL_BAR.h"
#include "STATIC.h"
#include "VEDIT.h"
#include "ERROR.h"

DIALOG_BOX::DIALOG_BOX (Display *dsp, WINDOW *prnt, unsigned long foreground,
   unsigned long background, char *res_name, Font fid, int *VParameters, 
   int *VID,char *name, unsigned int border_width) :
   WINDOW (dsp, prnt, DIALOG_BOX_NAME, name, 0, 0, 1, 1, border_width, 
   foreground, background, True, fid, foreground)
{
   Load (res_name, VParameters, VID);

   XSetWindowAttributes attr;

   attr .save_under = True;
   XChangeWindowAttributes (display, window, CWSaveUnder, &attr);
}

void DIALOG_BOX::Load (char *name, int *VParameters, int *VID)
{
   FILE *in = fopen (name, "r");

   if (in == NULL) {
      ERROR ("DBOX");
   }
   S0 (in, VParameters, VID);
   fclose (in);
}

void DIALOG_BOX::S0 (FILE *in, int *VParameters, int *VID)
{
   XFontStruct *font_info = QueryFont ();
   XSizeHints  hint;

   hint .x      = (int) (read_index (in) * MODX);
   hint .y      = (int) (read_index (in) * MODY);
   hint .width  = (int) (read_index (in) * MODX);
   hint .height = (int) (read_index (in) * MODY);

   SetNormalHints (&hint);
   MoveResize (hint .x, hint .y, hint .width, hint .height);
   for (;;) {
      char token [128];

      read_token (in, token);
      if (feof (in)) break;
      if (strcmp (token, "STATIC") == 0) {
         read_string (in, token);

                  int index  = read_index (in);
                  int x      = (int) (read_index (in) * MODX);
                  int y      = (int) (read_index (in) * MODY);
         unsigned int width  = (int) (read_index (in) * MODX);
         unsigned int height = (int) (read_index (in) * MODY);
         STATIC       *win   = new STATIC (display, this, token, x, y, width,
            height, index);
         
         if (win == NULL) {
            ERROR ("no memory");
         }
      } else if (strcmp (token, "EDIT") == 0) {
                  int index  = read_index (in);
                  int x      = (int) (read_index (in) * MODX);
                  int y      = (int) (read_index (in) * MODY);
         unsigned int width  = (int) (read_index (in) * MODX);
         unsigned int height = (int) (read_index (in) * MODY);
         EDIT         *win   = new EDIT (display, this, x, y, width, height,
            index);

         if (win == NULL) {
            ERROR ("no memory");
         }
      } else if (strcmp (token, "BUTTON") == 0) {
         read_string (in, token);

                  int index  = read_index (in);
                  int x      = (int) (read_index (in) * MODX);
                  int y      = (int) (read_index (in) * MODY);
         unsigned int width  = (int) (read_index (in) * MODX);
         unsigned int height = (int) (read_index (in) * MODY);
         BUTTON       *win   = new BUTTON (display, this, token, x, y, width,
            height, index);

         if (win == NULL) {
            ERROR ("no memory");
         }
      } else if (strcmp (token, "LIST_BOX") == 0) {
                  int index  = read_index (in);
                  int x      = (int) (read_index (in) * MODX);
                  int y      = (int) (read_index (in) * MODY);
         unsigned int width  = (int) (read_index (in) * MODX);
         unsigned int height = (int) (read_index (in) * MODY);
         LIST_BOX     *win   = new LIST_BOX (display, this, x, y, width, 
            height, index);

         if (win == NULL) {
            ERROR ("no memory");
         }
      } else if (strcmp (token, "SCROLL_BAR") == 0) {
                  int index  = read_index (in);
                  int x      = (int) (read_index (in) * MODX);
                  int y      = (int) (read_index (in) * MODY);
         unsigned int width  = (int) (read_index (in) * MODX);
         unsigned int height = (int) (read_index (in) * MODY);
         SCROLL_BAR   *win   = new SCROLL_BAR (display, this, x, y, width,
            height, index);

         if (win == NULL) {
            ERROR ("no memory");
         }
      } else if (strcmp (token, "VEDIT") == 0) {
         read_string (in, token);

         int n     = read_index (in);
         int index = read_index (in);

         if (n == -1) {
            int i = 0;

            for (;;) {
               if (VID [i] == index) break;
               i++;
            }
            n = VParameters [i];
         }

                  int x      = (int) (read_index (in) * MODX);
                  int y      = (int) (read_index (in) * MODY);
         unsigned int width  = (int) (read_index (in) * MODX);
         unsigned int height = (int) (read_index (in) * MODY);
         VEDIT       *win   = new VEDIT (display, this, token, x, y, width,
            height, index, n);

         if (win == NULL) {
            ERROR ("DBOX");
         }
      }
   }
   XFreeFontInfo (NULL, font_info, 1);
}

DIALOG_BOX::~DIALOG_BOX (void)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);

      char *name = ((WINDOW *) prop ->tc) ->ClassName ();

      if (strcmp (name, STATIC_NAME) == 0)
         delete (STATIC *) prop ->tc;
      else if (strcmp (name, EDIT_NAME) == 0)
         delete (EDIT *) prop ->tc;
      else if (strcmp (name, BUTTON_NAME) == 0)
         delete (BUTTON *) prop ->tc;
      else if (strcmp (name, LIST_BOX_NAME) == 0)
         delete (LIST_BOX *) prop ->tc;
      else if (strcmp (name, SCROLL_BAR_NAME) == 0)
         delete (SCROLL_BAR *) prop ->tc;
      else if (strcmp (name, VEDIT_NAME) == 0)
         delete (VEDIT *) prop ->tc;
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}

void DIALOG_BOX::Dialog (void)
{
   Show (True);
   for (;;) {
      XEvent event;

      XNextEvent (display, &event);
      if (Dispatch (&event) == EXIT_DBOX) break;
   }
   Show (False);
   XSync (display, True);
}
