/* File: EDIT.C */

#include <X11/keysym.h>
#include <stdio.h>
#include "EDIT.h"
#include "const.h"
#include "ERROR.h"

#define point_width  5
#define point_height 4

EDIT::EDIT (Display *dsp, WINDOW *prnt, int x, int y, unsigned int width,
   unsigned int height, int Index) :
   CONTROL (dsp, prnt, EDIT_NAME, NULL, x, y, width, height, 1,
   prnt ->GetForeground (), Index)
{
   int depth = DisplayPlanes (display, DefaultScreen (display));

   point = XCreatePixmap (display, window, point_width, point_height, 
      depth);
   if (point == None) {
      ERROR ("EDIT");
   }

   unsigned long foreground = GetForeground ();
   unsigned long background = GetBackground ();

   SetForeground (background);
   XFillRectangle (display, point, gc, 0, 0, point_width, point_height);
   SetForeground (foreground);
   XDrawLine (display, point, gc, 0, 2, 2, 0);
   XDrawLine (display, point, gc, 0, 3, 2, 1);
   XDrawLine (display, point, gc, 2, 0, 4, 2);
   XDrawLine (display, point, gc, 2, 1, 4, 3);
   SetText (NULL);
   SetInput (ExposureMask | KeyPressMask | EnterWindowMask | LeaveWindowMask);
}

void EDIT::Draw (char *name)
{
   XWindowAttributes xwa;

   GetAttributes (&xwa);
   DrawString (1, xwa .height - 2, name);
}

int EDIT::Go (XEvent *event)
{
   switch (event ->type) {
   case Expose:
      if (event ->xexpose .count == 0) Draw (buf);
      break;
   case KeyPress:
      {
         KeySym keysym;
         
         RemovePoint ();
         XLookupString (&event ->xkey, buf + pbuf, 1, &keysym, NULL);
         if (keysym == XK_BackSpace) {
            if (pbuf > 0) {
               buf [--pbuf] = '\0';
               Clear ();
            }
         } else {
            buf [++pbuf] = '\0';
         }
         Draw (buf);
         PutPoint ();
      }
      break;
   case EnterNotify:
      PutPoint ();
      break;
   case LeaveNotify:
      {
         RemovePoint ();

         XEvent nevent;

         nevent .type                 = ClientMessage;
         nevent .xclient .display     = display;
         nevent .xclient .window      = parent ->WindowID ();
         nevent .xclient .format      = 32;
         nevent .xclient .data .l [0] = (long) this;
         nevent .xclient .data .l [1] = (long) index;
         nevent .xclient .data .l [2] = (long) buf;
         SendEvent (&nevent);
      }
      break;
   }
   return NOTHING;
}

void EDIT::SetText (char *name)
{
   if (name == NULL) {
      buf [0] = '\0';
      pbuf    = 0;
   } else {
      strcpy (buf, name);
      pbuf = strlen (name);
   }
}

void EDIT::DisplayItself (void)
{
   Clear ();
   Draw (buf);
}

void EDIT::RemovePoint (void)
{
   XFontStruct       *font_info = QueryFont ();
   XWindowAttributes xwa;

   GetAttributes (&xwa);
   XClearArea (display, window, 1 + TRUE_TEXT_WIDTH (buf), xwa .height - 5, 
      point_width, point_height, False);
   XFreeFontInfo (NULL, font_info, 1);
}

void EDIT::PutPoint (void)
{
   XFontStruct       *font_info = QueryFont ();
   XWindowAttributes xwa;

   GetAttributes (&xwa);
   XCopyArea (display, point, window, gc, 0, 0, point_width, point_height,
      1 + TRUE_TEXT_WIDTH (buf), xwa .height - 5);
   XFreeFontInfo (NULL, font_info, 1);
}

EDIT::~EDIT (void)
{
    XFreePixmap (display, point);
}
