/* File: FLEXI.C */

#include "FLEXI.h"
#include "task.h"
#include "res.h"
#include "PAR_DBOX.h"
#include "ERROR.h"

#define DBOX_BACKGROUND WhitePixel (display, DefaultScreen (display))
#define DBOX_FOREGROUND BlackPixel (display, DefaultScreen (display))

METHODFLEXI::METHODFLEXI (double *Start, double *Lower, double *Upper, int N)
{
   start                          = Start;
   lower_bounds                   = Lower;
   upper_bounds                   = Upper;
   number_of_variables            = N;
   max_number_of_fun_eval         = 200;
   size_of_initial_polyhedron     = 0.3;
   desired_convergence            = 0.1e-6;
   number_of_constraints          = 0;
   number_of_equality_constraints = 0;
}

int METHODFLEXI::EnterParameters (Display *display, WINDOW *wnd)
{
   VELEMENT Variable [4];

   Variable [0] .ident   = START;
   Variable [0] .size    = number_of_variables;
   Variable [0] .pointer = (void *) start;
   Variable [0] .format  = DOUBLE;
   Variable [1] .ident   = XL;
   Variable [1] .size    = number_of_variables;
   Variable [1] .pointer = (void *) lower_bounds;
   Variable [1] .format  = DOUBLE;
   Variable [2] .ident   = XU;
   Variable [2] .size    = number_of_variables;
   Variable [2] .pointer = (void *) upper_bounds;
   Variable [2] .format  = DOUBLE;
   Variable [3] .pointer = NULL;

   int VPar [3];

   VPar [0] = Variable [0] .size;
   VPar [1] = Variable [1] .size;
   VPar [2] = Variable [2] .size;


   int VID [3];

   VID [0] = Variable [0] .ident;
   VID [1] = Variable [1] .ident;
   VID [2] = Variable [2] .ident;

   FELEMENT Fixed [6];

   Fixed [0] .ident   = FNMAX;
   Fixed [0] .pointer = (void *) &max_number_of_fun_eval;
   Fixed [0] .format  = INT;
   Fixed [1] .ident   = M;
   Fixed [1] .pointer = (void *) &number_of_constraints;
   Fixed [1] .format  = INT;
   Fixed [2] .ident   = ME;
   Fixed [2] .pointer = (void *) &number_of_equality_constraints;
   Fixed [2] .format  = INT;
   Fixed [3] .ident   = DELTA;
   Fixed [3] .pointer = (void *) &size_of_initial_polyhedron;
   Fixed [3] .format  = DOUBLE;
   Fixed [4] .ident   = EPS;
   Fixed [4] .pointer = (void *) &desired_convergence;
   Fixed [4] .format  = DOUBLE;
   Fixed [5] .pointer = NULL;

   PAR_DBOX *dbox = new PAR_DBOX (display, wnd, DBOX_FOREGROUND,
      DBOX_BACKGROUND, "flexip.res", Fixed, Variable, VPar, VID);

   if (dbox == NULL) {
      ERROR ("PAR_DBOX");
   }
   dbox ->Dialog ();
   delete dbox;
   return max_number_of_fun_eval;
}

void METHODFLEXI::Run (double *x, int n, double *y, 
   double (*fi) (const double *x, int n), 
   void (*constr) (const double *x, int n, double *g, int m),
   double *fm, double **xn)
{
   flexi (x, n, y, fi, constr, max_number_of_fun_eval,
      number_of_equality_constraints, number_of_constraints -
      number_of_equality_constraints, size_of_initial_polyhedron,
      desired_convergence, fm, xn);
}
