/* File: LBOX.C */

#include "const.h"
#include "work.h"
#include "LBOX.h"
#include "LBOX_ITEM.h"
#include "SCROLL_BAR.h"
#include "ERROR.h"

LIST_BOX::LIST_BOX (Display *dsp, WINDOW *prnt, int x, int y,
   unsigned int width, unsigned int height, int Index) :
   CONTROL (dsp, prnt, LIST_BOX_NAME, NULL, x, y, width, height, 1, 
   prnt ->GetForeground (), Index)
{
   selected = NULL;
   n_item   = 0;

   XFontStruct *font_info = QueryFont ();
   SCROLL_BAR  *win       = new SCROLL_BAR (display, this, 
      width - CHAR_HEIGHT - 1, -1, CHAR_HEIGHT, height, 1);

   if (win == NULL) {
      ERROR ("no memory");
   }   
   XFreeFontInfo (NULL, font_info, 1);
}

int LIST_BOX::Go (XEvent *event)
{
   switch (event ->type) {
   case ClientMessage:
      if (strcmp (((WINDOW *) event ->xclient .data .l [0]) ->ClassName (), 
         LIST_BOX_ITEM_NAME) == 0) {
         XEvent nevent;

         nevent .type                 = ClientMessage;
         nevent .xclient .display     = display;
         nevent .xclient .window      = parent ->WindowID ();
         nevent .xclient .format      = 32;
         nevent .xclient .data .l [0] = (long) this;
         nevent .xclient .data .l [1] = (long) index;
         if (selected) selected ->Select (False);
         selected = (LIST_BOX_ITEM *) event ->xclient .data .l [0];
         selected ->Select (True);

         char *name = selected ->GetName ();

         strcpy (buf, name);
         XFree (name);
         nevent .xclient .data .l [2] = (long) buf;
         SendEvent (&nevent);
      } else
         Scroll ((Direction) event ->xclient .data .l [1]);
      break;
   }
   return NOTHING;
}

void LIST_BOX::AddItem (char *string)
{
   XFontStruct       *font_info = QueryFont ();
   XWindowAttributes xwa;

   GetAttributes (&xwa);

   LIST_BOX_ITEM *win       = new LIST_BOX_ITEM (display, this, string, 0, 
      n_item * CHAR_HEIGHT, xwa .width - CHAR_HEIGHT, CHAR_HEIGHT);

   if (win == NULL) {
      ERROR ("no memory");
   }
   n_item++;
   XFreeFontInfo (NULL, font_info, 1);
}

LIST_BOX::~LIST_BOX (void)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);
      if (strcmp (((WINDOW *) prop ->tc) ->ClassName (), LIST_BOX_ITEM_NAME)
         == 0) 
         delete (LIST_BOX_ITEM *) prop ->tc;
      else
         delete (SCROLL_BAR *) prop ->tc;
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}

void LIST_BOX::Scroll (Direction up)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   if (selected) selected ->Select (False);
   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);
      if (strcmp (((WINDOW *) prop ->tc) ->ClassName (), LIST_BOX_ITEM_NAME) 
         == 0) {
         LIST_BOX_ITEM     *item = (LIST_BOX_ITEM *) prop ->tc;
         XWindowAttributes xwa;
         XSizeHints        hint;

         item ->GetAttributes (&xwa);
         item ->Show (False);
         hint .x       = xwa .x;
         hint .y      += (up == Up ? 1 : -1) * xwa .height;
         hint .width   = xwa .width;
         hint .height  = xwa .height;
         item ->SetNormalHints (&hint);
         item ->Move (hint .x, hint .y);
         item ->Show (True);
      }
      XFree ((char *) prop);
   }
   if (selected) selected ->Select (True);
   if (n_children > 0) XFree ((char *) children);
}
