/* File: LPMIN.C */

#include "LPMIN.h"
#include "task.h"
#include "PAR_DBOX.h"
#include "res.h"
#include "ERROR.h"

#define DBOX_BACKGROUND WhitePixel (display, DefaultScreen (display))
#define DBOX_FOREGROUND BlackPixel (display, DefaultScreen (display))

METHODLPMIN::METHODLPMIN (double *xl, double *xu, int n)
{
   lower_bounds             = xl;
   upper_bounds             = xu;
   number_of_variables      = n;
   number_of_iterations     = 1000;
   number_of_initial_points = 50;
}

int METHODLPMIN::EnterParameters (Display *display, WINDOW *wnd)
{
   VELEMENT Variable [3];

   Variable [0] .ident   = XL;
   Variable [0] .size    = number_of_variables;
   Variable [0] .pointer = (void *) lower_bounds;
   Variable [0] .format  = DOUBLE;
   Variable [1] .ident   = XU;
   Variable [1] .size    = number_of_variables;
   Variable [1] .pointer = (void *) upper_bounds;
   Variable [1] .format  = DOUBLE;
   Variable [2] .pointer = NULL;

   int VPar [2];

   VPar [0] = Variable [0] .size;
   VPar [1] = Variable [1] .size;

   int VID [2];

   VID [0] = Variable [0] .ident;
   VID [1] = Variable [1] .ident;

   FELEMENT Fixed [3];

   Fixed [0] .ident   = IT;
   Fixed [0] .pointer = (void *) &number_of_iterations;
   Fixed [0] .format  = INT;
   Fixed [1] .ident   = LT;
   Fixed [1] .pointer = (void *) &number_of_initial_points;
   Fixed [1] .format  = INT;
   Fixed [2] .pointer = NULL;

   PAR_DBOX *dbox = new PAR_DBOX (display, wnd, DBOX_FOREGROUND,
      DBOX_BACKGROUND, "lpminp.res", Fixed, Variable, VPar, VID);

   if (dbox == NULL) {
      ERROR ("PAR_DBOX");
   }
   dbox ->Dialog ();
   delete dbox;
   return number_of_iterations;
}

void METHODLPMIN::Run (double *x, int n, double *y, 
   double (*fi) (const double *x, int n), double *fm, double **xn)
{
   lpmin (x, lower_bounds, upper_bounds, n, y, fi, number_of_iterations, 
      number_of_initial_points, fm, xn);
}
