/* File: SUBMENU.C */

#include "const.h"
#include "work.h"
#include "SUBMENU.h"
#include "SMENU_ITEM.h"
#include "ERROR.h"

void SUBMENU::S1 (unsigned int *width, unsigned int *height, FILE *in)
{
   int y = 0;

   *width = 0;
   for (;;) {
      char token [128];

      read_token (in, token);
      if (strcmp (token, "MENUITEM") == 0) {
         read_string (in, token);

         XFontStruct *font_info = QueryFont ();

         *width = MAX (*width, TEXT_WIDTH (token));

         SUBMENU_ITEM *submenu_item = new SUBMENU_ITEM (display, this, token,
            0, y, 1, 1, read_index (in));

         if (submenu_item == NULL) {
            ERROR ("no memory");
         }
         y += CHAR_HEIGHT;
         XFreeFontInfo (NULL, font_info, 1);
      } else
         break;
   }
   *height = y;
}

int SUBMENU::Go (XEvent *event)
{
   switch (event ->type) {
   case ClientMessage:
      Show (False);
      SetVisible (False);
      return (int) event ->xclient .data .l [0];
   case LeaveNotify:
      {
         XWindowAttributes xwa;

         GetAttributes (&xwa);
         if (event ->xcrossing .x <= 0          || 
             event ->xcrossing .x >= xwa .width ||
             event ->xcrossing .y >= xwa .height) {
            Show (False);
            SetVisible (False);
         }
      }
   }
   return NOTHING;
}

void SUBMENU::Justify (void)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);
      ((SUBMENU_ITEM *) (prop ->tc)) ->SUBMENU_ITEM::Justify ();
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}

SUBMENU::SUBMENU (Display *dsp, WINDOW *prnt, int x, int y, unsigned int width,
   unsigned int height, unsigned long border, unsigned long background) :
   WINDOW (dsp, prnt, SUBMENU_NAME, NULL, x, y, width, height, 1, border,
   background, False, prnt ->GetFont (), prnt ->GetForeground ())
{
   SetInput (LeaveWindowMask);

   XSetWindowAttributes attr;

   attr .save_under = True;
   XChangeWindowAttributes (display, window, CWSaveUnder, &attr);
}

SUBMENU::~SUBMENU (void)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);
      delete (SUBMENU_ITEM *) prop ->tc;
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}
