/* File: VEDIT.C */

#include "const.h"
#include "work.h"
#include "VEDIT.h"
#include "SCROLL_BAR.h"
#include "EDIT.h"
#include "STATIC.h"
#include "ERROR.h"

VEDIT::VEDIT (Display *dsp, WINDOW *prnt, char *name, int x, int y,
   unsigned int width, unsigned int height, int Index, int NItem) :
   CONTROL (dsp, prnt, VEDIT_NAME, NULL, x, y, width, height, 0, 
   prnt ->GetForeground (), Index)
{
   n_item = NItem;

   XFontStruct *font_info = QueryFont ();

   int sbwidth = 0;

   if (NItem * (CHAR_HEIGHT + 2) - 1 > height) {
      sbwidth = CHAR_HEIGHT + 1;

      SCROLL_BAR  *sb        = new SCROLL_BAR (display, this, 
         width - CHAR_HEIGHT - 2, 0, CHAR_HEIGHT, height - 2, NItem);

      if (sb == NULL) {
         ERROR ("VEDIT");
      }
   }

   int swidth = (width - CHAR_HEIGHT) / 2 - 1;

   for (int i = 0; i < NItem; i++) {
      char buf [128];

      sprintf (buf, "%s(%.1d)", name, i + 1);

      STATIC *stat = new STATIC (display, this, buf, 0, i * (CHAR_HEIGHT + 1), 
         swidth, CHAR_HEIGHT + 2, NItem + i + 1);

      if (stat == NULL) {
         ERROR ("VEDIT");
      }

      EDIT *edit = new EDIT (display, this, swidth, i * (CHAR_HEIGHT + 1), 
         width - swidth - sbwidth - 2, CHAR_HEIGHT, i);

      if (edit == NULL) {
         ERROR ("VEDIT");
      }
   }
   XFreeFontInfo (NULL, font_info, 1);
}

int VEDIT::Go (XEvent *event)
{
   switch (event ->type) {
   case ClientMessage:
      {
         char *name = ((WINDOW *) event ->xclient .data .l [0]) ->
             ClassName ();

         if (strcmp (name, EDIT_NAME) == 0) {
            XEvent nevent;

            nevent .type                 = ClientMessage;
            nevent .xclient .display     = display;
            nevent .xclient .window      = parent ->WindowID ();
            nevent .xclient .format      = 32;
            nevent .xclient .data .l [0] = (long) this;
            nevent .xclient .data .l [1] = (long) index;
            nevent .xclient .data .l [2] = event ->xclient .data .l [0];
            nevent .xclient .data .l [3] = event ->xclient .data .l [1];
            nevent .xclient .data .l [4] = event ->xclient .data .l [2];
            SendEvent (&nevent);
         } else if (strcmp (name, SCROLL_BAR_NAME) == 0) 
            Scroll ((Direction) event ->xclient .data .l [1]);
      }
      break;
   }
   return NOTHING;
}

VEDIT::~VEDIT (void)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);

      char *name = ((WINDOW *) prop ->tc) ->ClassName ();

      if (strcmp (name, STATIC_NAME) == 0) 
         delete (STATIC *) prop ->tc;
      else if (strcmp (name, EDIT_NAME) == 0) 
         delete (EDIT *) prop ->tc;
      else if (strcmp (name, SCROLL_BAR_NAME) == 0) 
         delete (SCROLL_BAR *) prop ->tc;
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}

void VEDIT::DisplayItself (void)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);

      char *name = ((WINDOW *) prop ->tc) ->ClassName ();

      if (strcmp (name, EDIT_NAME) == 0) 
         ((EDIT *) prop ->tc) ->DisplayItself ();
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}

void VEDIT::Scroll (Direction up)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   int max_height = 0;
   int min_height = 1;
   int i;
   for (i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);
      if (strcmp (((WINDOW *) prop ->tc) ->ClassName (), STATIC_NAME) == 0) {
         STATIC            *item = (STATIC *) prop ->tc;
         XWindowAttributes xwa;

         item ->GetAttributes (&xwa);
         max_height = MAX (max_height, xwa .y + xwa .height);
         min_height = MIN (min_height, xwa .y);
      }
      XFree ((char *) prop);
   }

   XFontStruct *font_info = QueryFont ();
   XWindowAttributes xwa;

   GetAttributes (&xwa);
   if ((max_height > xwa .height && up == Down) ||
       (min_height < 0           && up == Up)) {
      for (i = 0; i < n_children; i++) {
         struct {
            unsigned char *tc;
         } *prop;

         GetThisProperty (display, children [i], (unsigned char **) &prop);
         if (strcmp (((WINDOW *) prop ->tc) ->ClassName (), STATIC_NAME) 
            == 0) {
            STATIC *item = (STATIC *) prop ->tc;

            item ->GetAttributes (&xwa);
            item ->Show (False);
            item ->Move (xwa .x, 
               xwa .y + (up == Up ? 1 : -1) * (xwa .height - 1));
            item ->Show (True);
         } else if (strcmp (((WINDOW *) prop ->tc) ->ClassName (), EDIT_NAME)
            == 0) {
            EDIT *item = (EDIT *) prop ->tc;

            item ->GetAttributes (&xwa);
            item ->Show (False);
            item ->Move (xwa .x, 
               xwa .y + (up == Up ? 1 : -1) * (xwa .height + 1));
            item ->Show (True);
         }
         XFree ((char *) prop);
      }
   }
   XFreeFontInfo (NULL, font_info, 1);
   if (n_children > 0) XFree ((char *) children);
}

void VEDIT::SetText (int nItem, char *text)
{
   Window       *children;
   unsigned int n_children;

   GetChildren (&children, &n_children);

   for (int i = 0; i < n_children; i++) {
      struct {
         unsigned char *tc;
      } *prop;

      GetThisProperty (display, children [i], (unsigned char **) &prop);
      if (((CONTROL *) prop ->tc) ->ID () == nItem) 
         ((EDIT *) prop ->tc) ->SetText (text);
      XFree ((char *) prop);
   }
   if (n_children > 0) XFree ((char *) children);
}
