#/* File: FI.C */

#include "fi.h"
#include <math.h>
#include <stdio.h>

#define P 5 /* number of parameters a */
//#define P 2
//#define Q 10 /* number of parameters b */
#define Q 2
//#define T 252 /* number of data entries */
#define T 400
//#define T 4
//#define R 30 /* number of nonzero d-components */
#define R 10
#define DATAFILE "exchr"

static double z[T], w[T], d[R], A[T][P], B[T], ma[P][P], mb[P] ;
static double a[P], *b ;
int number_of_variables = Q+1 ;

void get_d (double dd) 
{
    for ( int i = 0 ; i < R ; i++ ) {
   	extern int signgam ;
	double mult = 1 ;
	for ( int j = 0 ; j < i ; j++ ) 
	    mult *= j - dd ;
	d[i] = mult / exp ( lgamma ((double)i+1) ) * signgam ;
    }
}

void get_z () 
{
    FILE* fp = fopen (DATAFILE,"r") ;
   for ( int t = 0 ; t < T ; t++ ) 
	fscanf ( fp, "%lg", &z[t] ) ;
    fclose (fp) ;
}

void get_w ()
{
    for (int t = 0 ; t < T ; t++ ) { 
        w[t] = z[t] ;
	for ( int j = 0 ; j < R && t-j > 0 ; j++ ) 
	    w[t] -= d[j]*z[t-j-1] ;
    }
}

void get_AB ()
{
    for ( int t = 0 ; t < T ; t++ ) 
	for ( int i = 0 ; i < P && t-i > 0 ; i++ ) {
	    A[t][i] = - w[t-i-1] ;
            for ( int j = 0 ; j < Q && t-j > 0 ; j++ )
		A[t][i] += b[j] * A[t-j-1][i] ;
	}
    for   ( t = 0 ; t < T ; t++ ) {
	B[t] = w[t] ; 
	for ( int j = 0 ; j < Q && t-j > 0 ; j++ )
		B[t] += b[j]*B[t-j-1] ;
    }
}

void get_ma_mb()
{
    for ( int i=0 ; i < P ; i++ ) {
	mb[i] = 0 ;
	for ( int j=0 ; j < P ; j++ ) {
            ma[i][j] = 0 ;
 	    for ( int t = 0 ; t < T ; t++)
		ma[i][j] += A[t][j] * A[t][i] ;
    	} 
	mb[i] = 0 ;
	for ( int t = 0 ; t < T ; t++)
	    mb[i] -= B[t] * A[t][i] ;
    }
}

#define AMIN 1e-100

void gauss()
{
    int i,j,max;
    double t;
    for(i=0;i<P-1;i++) {
	int k;
	for(j=i+1,max=i;j<P;j++) {
	    if( fabs(ma[j][i]) > fabs(ma[max][i]) )
		max = j ;
	}
	if ( max != i ) {
	    for(k=0;k<P;k++) {
		t = ma[max][k];
		ma[max][k] = ma[i][k];
		ma[i][k] = t;
	    }
	    t = mb[max];
	    mb[max] = mb[i];
	    mb[i] = t;
	}

	for ( j=i+1;j<P;j++) {
	    double s;
	    s=ma[j][i]/( fabs(ma[i][i]) > AMIN ? ma[i][i] : AMIN);
       /*   s=ma[j][i]/ma[i][i];   */
	    for ( k=P-1; k>=i; k--)
		ma[j][k] -= ma[i][k]*s ;
	    mb[j] -= mb[i]*s ;
	}
    }

    for(i = P-1;i>=0;i--) {
	a[i] = mb[i];
	for(j=P-1; j>i;j--)
	    a[i] -= ma[i][j]*a[j] ;
	a[i] /= (fabs(ma[i][i])>AMIN ? ma[i][i] : AMIN) ;
/*      a[i] /= ma[i][i];                         */
    }
}

double fm () {
    double sum = 0 , etsum = 0;
    static double minsum = 1e300 ;
//    FILE *fp = fopen ( "ftime.ser" , "w") ;
    for ( int t=0 ; t < T ; t++ ) {
	double et = B[t] ;
	for ( int j=0 ; j < P ; j++ ) 
	    et += A[t][j]*a[j] ;
	sum += et*et ; etsum -= et-w[t] ;
//	fprintf ( fp, "%g\n", etsum) ;
    }		      
//    fclose (fp) ;
    if ( sum < minsum ) {
	minsum = sum ;
//	rename ( "ftime.ser", "ftime.best" ) ;
    }		  
    return sum ;
}

/* Goal function definition */
/* First Q x-ses are b's, last one is d */

double fi (const double *x, int n)
{
    static int first = 1 ;
    static double dd ;
    b = x ;

//printf ("Entering fi\n");
    for ( int i=0 ; i < Q ; i++)
//	printf ( "b[%d]=%e\n",i+1,b[i]);
//    printf ( "d=%e\n",b[Q] ) ;
    if (first) {
	dd = x[Q] ;
	first = 0 ;
        get_d (dd) ;
        get_z () ;
    } 
    else if ( dd != x[Q] ) { 
        dd = x[Q] ;  
	get_d (dd) ;
    }
    get_w () ;
    get_AB () ;
    get_ma_mb () ;
    gauss () ;
//    for ( i=0 ; i < P ; i++ )
//	printf ( "a[%d]=%e\n",i+1,a[i]);
    double res = fm () ;
//printf ( "fm=%e\n",res) ;
    return log (res) ;
//    return res ;
}

void constr (const double *x, int n, double *g, int ng)
{
/*
	ng > 0 - calculates all constaints
	ng < 0  - calculates only constraint with number (-ng)
*/
/*   double x1 = x [0];
   double x2 = x [1];
   double x3 = x1 * x1;
   double x4 = x2 * x2;

   if (ng > 0) {
		g [0] = 25. - x3 - x4;
		g [1] = 10. * x1 - x3 + 10. * x2 - x4 - 34.;
		g [2] = x1;
		g [3] = x2;
   } else {
      ng = -ng;
      switch (ng) {
      case 1:
			g [0] = 25. - x3 - x4;
			break;
      case 2:
			g [1] = 10. * x1 - x3 + 10. * x2 - x4 - 34.;
			break;
      case 3:
			g [2] = x1;
			break;
      case 4:
			g [3] = x2;
			break;
      }
   } */
   if ( ng > 0 ) 
	for ( int i = 0 ; i < n ; i++ ) g[i] = 0 ;
   else g[-ng] = 0 ;

}

