/* File: FI.C */

#include "fi.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

/* Goal function definition */

int number_of_variables = 2;

#include "task.h"

#define inf                10000.
#define NUMBER_OF_STEPS    10
#define NUMBER_OF_JOBS     6
#define NUMBER_OF_MACHINES 10
#define DIM                3
#define IT                 100
#define LT                 5
#define NUMBER_OF_OPT      10
#define NUMBER_OF_FCALC    1
#define FALSE              0
#define TRUE               1

class decision {
   private:
      int    job, machine;
      double time;
   public:
      int & Job (void) {return job;};
      int & Machine (void) {return machine;};
      double & Time (void) {return time;};
   };

class step {
   private:
      int    machine;
      double time;
   public:
      step (void) {};
      void operator = (const step B) {machine = B. machine; time = B .time;};
      step (int m, double t) {machine = m; time = t;};
      int Machine (void) {return machine;};
      double Time (void) {return time;};
   };

class job {
   private:
      step stp [128];
      int  number_of_steps;
      int  first_unused_step;
   public:
      job (void) {number_of_steps = 0;};
      void Add (const step s) {stp [number_of_steps++] = s;};
      step & FirstUnusedStep (void) {return stp [first_unused_step];};
      int IsFinished (void) {
         return (number_of_steps == first_unused_step) ? TRUE : FALSE;
      };
      double TimeLeft (void) {
         double t = 0.;

         for (int i = first_unused_step; i < number_of_steps; i++) {
            t += stp [i] .Time ();
         }
         return t;
      }
      void UseStep (void) {first_unused_step++;};
      void Init (void) {first_unused_step = 0;}
   };

static void normalize (const double *xx, int dim, double *x);
static void do_selection (decision *D, job *j, int selection, 
   int number_of_jobs, double *machine_time, double *job_time);
static double fif (const double *xx, int dim);

int S [NUMBER_OF_JOBS];

int P [NUMBER_OF_JOBS] [NUMBER_OF_STEPS];

double T [NUMBER_OF_JOBS] [NUMBER_OF_STEPS];

double fi (const double *xx, int dim)
{
   double x [DIM];

   normalize (xx, dim, x);

   double fmin = r1mach (2);

   for (int k = 0; k < NUMBER_OF_FCALC; k++) { 
      double s = fif (x, dim);

      fmin = MIN (s, fmin);
   }
   return fmin;
}

static double fif (const double *xx, int dim)
{
   static int      init = FALSE;
   static job      j [NUMBER_OF_JOBS];

   if (init == FALSE) {
      for (int l = 0; l < NUMBER_OF_JOBS; l++) {
         for (int i = 0; i < S [l]; i++) {
            j [l] .Add (step (P [l] [i], T [l] [i]));
         }
      }
      init  = TRUE;
   }

   decision D [NUMBER_OF_JOBS];
   double   machine_time [NUMBER_OF_MACHINES];
   double   job_time     [NUMBER_OF_JOBS];

   for (int i = 0; i < NUMBER_OF_JOBS; i++) {
      j [i] .Init ();
      job_time [i] = 0.;
   }
   for (i = 0; i < NUMBER_OF_MACHINES; i++) 
      machine_time [i] = 0.;
   for (;;) {
      int n = 0;
      int selection;

      for (int i = 0; i < NUMBER_OF_JOBS; i++) {
         if (j [i] .IsFinished () == TRUE) 
            n++;
         else
            selection = i;
      }
      if (n == NUMBER_OF_JOBS) break;
      if (n == NUMBER_OF_JOBS - 1) {
         while (j [selection] .IsFinished () == FALSE)
            do_selection (D, j, selection, NUMBER_OF_JOBS, machine_time, 
               job_time);
         break;
      }

      double minA = r1mach (2);
      double maxA = -r1mach (2);

      for (i = 0; i < NUMBER_OF_JOBS; i++) {
         if (j [i] .IsFinished () == TRUE) continue;
         minA = MIN (minA, j [i] .TimeLeft ());
         maxA = MAX (maxA, j [i] .TimeLeft ());
      }

      double h [NUMBER_OF_JOBS];
      double maxh = r1mach (2);
      int    k;

      for (i = 0; i < NUMBER_OF_JOBS; i++) {
         if (j [i] .IsFinished () == TRUE) continue;
         if (minA == maxA) 
            h [i] = 1.;
         else
            h [i] = (j [i] .TimeLeft () - minA) / (maxA - minA);
         if (h [i] < maxh) {
            maxh = h [i];
            k    = i;
         }
      }

      double a2 = 0.;

      for (i = 0; i < NUMBER_OF_JOBS; i++) {
         if (j [i] .IsFinished () == TRUE) continue;
         a2 += h [i];
      }
      a2 = 1. / a2;

      double a3 = 0.;

      for (i = 0; i < NUMBER_OF_JOBS; i++) {
         if (j [i] .IsFinished () == TRUE) continue;
         a3 += h [i] * h [i];
      }
      a3 = 1. / a3;

      double eps = ats ();
      double r   = 0.;

      for (i = 0; i < NUMBER_OF_JOBS; i++) {
         if (j [i] .IsFinished () == TRUE) continue;
         r += 1. / (NUMBER_OF_JOBS - n) * xx [0] + a2 * xx [1] * h [i] + 
            a3 * xx [2] * h [i] * h [i];
         if (r > eps) {
//fprintf (output, "selection %d\n", i);
            do_selection (D, j, i, NUMBER_OF_JOBS, machine_time, 
               job_time);
            break;
         }
      }
   }

   double max = 0.;

   for (i = 0; i < NUMBER_OF_JOBS; i++)
      max = MAX (max, job_time [i]);
   return max;
}

static void do_selection (decision *D, job *j, int selection, 
   int number_of_jobs, double *machine_time, double *job_time)
{
   for (int i = 0; i < number_of_jobs; i++) {
      D [i] .Job ()     = i;
      D [i] .Machine () = j [i] .FirstUnusedStep () .Machine ();
      D [i] .Time ()    = j [i] .FirstUnusedStep () .Time ();
   }
 
   decision d = D [selection];

   job_time [d .Job ()] = machine_time [d .Machine ()] =
      MAX (job_time [d .Job ()], machine_time [d .Machine ()]) +
      j [d .Job ()] .FirstUnusedStep () .Time ();
   j [d .Job ()] .UseStep ();
}

static void normalize (const double *xx, int dim, double *x)
{
   double sum = 0.;

   for (int i = 0; i < dim; i++) sum += xx [i];
   for (i = 0; i < dim; i++) x [i] = xx [i] / sum;
}

/* Definition of constraints */

void constr (const double *x, int n, double *g, int ng)
{
/*
	ng > 0 - calculates all constaints
	ng < 0  - calculates only constraint with number (-ng)
*/
   double x1 = x [0];
   double x2 = x [1];
   double x3 = x1 * x1;
   double x4 = x2 * x2;

   if (ng > 0) {
		g [0] = 25. - x3 - x4;
		g [1] = 10. * x1 - x3 + 10. * x2 - x4 - 34.;
		g [2] = x1;
		g [3] = x2;
   } else {
      ng = -ng;
      switch (ng) {
      case 1:
			g [0] = 25. - x3 - x4;
			break;
      case 2:
			g [1] = 10. * x1 - x3 + 10. * x2 - x4 - 34.;
			break;
      case 3:
			g [2] = x1;
			break;
      case 4:
			g [3] = x2;
			break;
      }
   }
}
