#/* File: FI.C */

#include "fi.h"
#include <math.h>
#include <stdio.h>
#include "fitimeb.h"

static double z[T0], A[T0][P], B[T0], c[S][R], ma[P][P], mb[P] ;
static double a[P], *b ;
int number_of_variables = Q + S*R  ;

void get_z () 
{
    FILE* fp = fopen (DATAFILE,"r") ;
    for ( int t = 0 ; t < T0 ; t++ ) 
	fscanf ( fp, "%lg", &z[t] ) ;
    fclose (fp) ;
}

void get_AB ()
{
    for ( int t = 0 ; t < T0 ; t++ )
	for ( int i = 0 ; i < P ; i++ )
	    A[t][i] = 0 ;
    for ( t = 0 ; t < P ; t++ ) 
	B[t] = 0 ;

    for ( t = 0 ; t < T0 ; t++ ) 
	for ( int i = 0 ; i < P && t-i > 0 ; i++ ) {
	    A[t][i] = - z[t-i-1] ;
            for ( int j = 0 ; j < Q && t-j > 0 ; j++ )
		A[t][i] += b[j] * A[t-j-1][i] ;
	    for ( int k = 0 ; k < S && t-k > 0 ; k++ )
		for ( int l = 0 ; l < R && t-l > 0 ; l++ )
		    A[t][i] -= c[k][l]*z[t-k-1]*A[t-l-1][i] ;
	}
    for ( t = 0 ; t < T0 ; t++ ) {
	B[t] = z[t] ; 
	for ( int j = 0 ; j < Q && t-j > 0 ; j++ ) 
	    B[t] += b[j]*B[t-j-1] ;
        for ( int k = 0 ; k < S && t-k > 0 ; k++ )
	    for ( int l = 0 ; l < R && t-l > 0 ; l++ )
		B[t] -= c[k][l]*z[t-k-1]*B[t-l-1];
    }
}

void get_ma_mb()
{
    for ( int i=0 ; i < P ; i++ ) {
	mb[i] = 0 ;
	for ( int j=0 ; j < P ; j++ ) {
            ma[i][j] = 0 ;
 	    for ( int t = 0 ; t < T0 ; t++)
		ma[i][j] += A[t][j] * A[t][i] ;
    	} 
	mb[i] = 0 ;
	for ( int t = 0 ; t < T0 ; t++)
	    mb[i] -= B[t] * A[t][i] ;
    }
}

#define AMIN 1e-100

void gauss()
{
    int i,j,max;
    double t;
    for(i=0;i<P-1;i++) {
	int k;
	for(j=i+1,max=i;j<P;j++) {
	    if( fabs(ma[j][i]) > fabs(ma[max][i]) )
		max = j ;
	}
	if ( max != i ) {
	    for(k=0;k<P;k++) {
		t = ma[max][k];
		ma[max][k] = ma[i][k];
		ma[i][k] = t;
	    }
	    t = mb[max];
	    mb[max] = mb[i];
	    mb[i] = t;
	}

	for ( j=i+1;j<P;j++) {
	    double s;
	    s=ma[j][i]/( fabs(ma[i][i]) > AMIN ? ma[i][i] : AMIN);
       /*   s=ma[j][i]/ma[i][i];   */
	    for ( k=P-1; k>=i; k--)
		ma[j][k] -= ma[i][k]*s ;
	    mb[j] -= mb[i]*s ;
	}
    }

    for(i = P-1;i>=0;i--) {
	a[i] = mb[i];
	for(j=P-1; j>i;j--)
	    a[i] -= ma[i][j]*a[j] ;
	a[i] /= (fabs(ma[i][i])>AMIN ? ma[i][i] : AMIN) ;
/*      a[i] /= ma[i][i];                         */
    }
}

double fm () {
    double sum = 0 , etsum = 0;
    for ( int t=1 ; t < T0 ; t++ ) {
	double et = B[t] ;
	for ( int j=0 ; j < P ; j++ ) 
	    et += A[t][j]*a[j] ;
	sum += et*et ; etsum -= et-z[t] ;
    }		      
    return sum ;
}

/* Goal function definition */
/* First Q x-ses are b's, last one is d */

double fi (const double *x, int n)
{
    static int first = 1 ;
    static double min = 1e300 ;
    FILE *fp ;
    b = x ;

    if ( first ) {
	get_z () ;
	first = 0 ;
    }
    get_AB () ;
    get_ma_mb () ;
    gauss () ;
    double res = fm () ;
    if ( res < min ) {
	min = res ;
	fp = fopen ( "time2/optimal.C", "w") ;
	
	fprintf ( fp, "#include \"fitimeb.h\"\n" ) ;
	fprintf ( fp, "double a[P] = { %e", a[0]) ;

        for ( int i=1 ; i < P ; i++ )
	    fprintf ( fp, ", %e",a[i]);

	fprintf ( fp, "} ,\n\tb[Q] = { %e", b[0] ) ;

    	for ( i=1 ; i < Q ; i++)
	    fprintf ( fp, ", %e",b[i]);

	fprintf ( fp, "} ,\n\tc[S][R] ") ;

	if ( S > 0 && R > 0 ) {
	    for (int k = 0 ; k < S ; k++) 
		for (int l = 0 ; l < R ; l++)
		    c[k][l]=x[S+k*R+l];

	    fprintf ( fp, "= {\n\t") ;
	    for ( k = 0 ; k < S ; k++) {
		fprintf ( fp, "{ %e", c[k][0] ) ;
		for (int l = 1 ; l < R ; l++)
		    fprintf ( fp, ", %e", c[k][l] ) ;
		fprintf ( fp, " }\n\t" ) ;
	    }
	    fprintf ( fp, "} " ) ;
	}
	fprintf ( fp, ";\n /* fm=%e*/\n",res) ;
	fclose (fp) ;
    }

    return log (res) ;
//    return res ;
}

void constr (const double *x, int n, double *g, int ng)
{
/*
	ng > 0 - calculates all constaints
	ng < 0  - calculates only constraint with number (-ng)
*/
/*   double x1 = x [0];
   double x2 = x [1];
   double x3 = x1 * x1;
   double x4 = x2 * x2;

   if (ng > 0) {
		g [0] = 25. - x3 - x4;
		g [1] = 10. * x1 - x3 + 10. * x2 - x4 - 34.;
		g [2] = x1;
		g [3] = x2;
   } else {
      ng = -ng;
      switch (ng) {
      case 1:
			g [0] = 25. - x3 - x4;
			break;
      case 2:
			g [1] = 10. * x1 - x3 + 10. * x2 - x4 - 34.;
			break;
      case 3:
			g [2] = x1;
			break;
      case 4:
			g [3] = x2;
			break;
      }
   } */
   if ( ng > 0 ) 
	for ( int i = 0 ; i < n ; i++ ) g[i] = 0 ;
   else g[-ng] = 0 ;
}
