/* File: glopt.C */

#include <stdio.h>
#include <malloc.h>
#include <values.h>
#include <stdlib.h>
#include <math.h>
#include "const.h"
#include "res.h"
#include "task.h"
#include "ERROR.h"
#include "OUTPUT.h"

static union {
   struct {
      int ifail, it, itm, fn;
   } _1;
   struct {
      int ifail, nr, itm, l;
   } _2;
} statis_;

#define statis_1 (statis_._1)
#define statis_2 (statis_._2)

static void glopt_(double *xm, const double *a, const double *b, int *m, 
   double *fm, 
   double (*f) (const double *x, int n), int cnmax, int pn, int fnmax, 
   int *nd, int *reduc, double *c, int *markm,  
   double *stplx, int *fnp, int *pg, double *wr1, double *x, int *m1, 
   int *wi1, double *wr2, double *wr3, double *wr4, double *wr5, double *s, 
   int *wi2, int *mark, int *pnc, int *nextp, int *lastp);
static void eigval_(int *lp, double *e, double *a, double *b, double *w, 
   double *f);
static void tridmx_(int *n, int *nm, double *a, double *d, double *b);
static void glopo_(int *m, int *ndp, int *n, int *guess, const double *a, 
   double *c, 
   int *nd, int *pg, double *x, int *step, double *r, double *z, int *m1,
   double (*f) (const double *x, int n));
static void clust_(int *fenmx, int *m, int *cc, int *fen, int *n, double *stpl,
   double *stplx, int *pg, int *fenp, double *x, double *y, double *z, 
   double *s, int *m1, const double *a, double *c, int *m2,
   double (*f) (const double *x, int n));
static void locop_(int *m, double *y, double *sf, double *sl, double *sll, 
   int *new_, 
   int *fen, int *fenmx, double *x, double *s, int *m1, const double *a, 
   double *c,
   double (*f) (const double *x, int n));
static void radus_(int *dim, int *dimax, int *n, double *alfa, double *x, 
   int *p, 
   int *d, double *r1, double *s, double *a, double *b, double *e, double *w, 
   double *f, int *m1);
static void msort_(int *n, double *a, int *list, int *start, int *hds);
static void separ_(int *dim, int *n, int *d, double *r1, int *p, double *x, 
   int *ncl, 
   int *mark, int *markr, double *c, double *r, int *nmcv, int *distp, 
   double *dist, int *m1, int *nextp, int *lastp);
static void outre_(int *m, int *sn, int *pn, int *cn, int *reduc, int *pg, 
   int *markm,
   int *mark, int *pnc);
static void undev_(double *a);
static void normd_(double *a);
static int ira_(double *x, const double *a, double *c, int *m);
static int ispa16_(int *nc, int *pnc, int *mark, int *n, double *x, double *fm,
   double *xm, int *m1sp);

static double *fafm;
static double **faxn;

void glopt (double *xm, const double *a, const double *b, int m, double *fm, 
   double (*f) (const double *x, int m), int cnmax, int pn, int fnmax, 
   double *pfm, double **pxn)
{
   int    *nd = NULL;
   int    *reduc, *markm, *fnp, *pg;
   int    *wi1, *wi2, *mark, *pnc, *nexp, *lastp;
   int    m1;
   double *c = NULL;
   double *stplx, *wr1;
   double *x, *wr2, *wr3, *wr4, *wr5, *s;

   fafm  = pfm;
   faxn  = pxn;
   m1    = m + 1;
   if ((nd = (int *) malloc ((unsigned int) sizeof (int) *
      (m + cnmax + 9 * pn))) == NULL) {
      ERROR ("out of memory", False);
      goto ret;
   }
   reduc = nd + m;
   markm = reduc + cnmax;
   fnp   = markm + pn;
   pg    = fnp + pn;
   wi1   = pg + pn;
   wi2   = wi1 + pn;
   mark  = wi2 + pn;
   pnc   = mark + pn;
   nexp  = pnc + pn;
   lastp = nexp + pn;
   if ((c = (double *) malloc ((unsigned int) sizeof (double) *
      (m + pn + m1 * pn + 2 * pn + 3 * m1 + m * m))) == NULL) {
      ERROR ("out of memory", False);
      goto ret;
   }
   stplx = c + m;
   x     = stplx + pn;
   wr1   = x + m1 * pn;
   wr2   = wr1 + pn;
   wr3   = wr2 + pn;
   wr4   = wr3 + m1;
   wr5   = wr4 + m1;
   s     = wr5 + m1;
   glopt_ (xm, a, b, &m, fm, f, cnmax, pn, fnmax, nd, reduc, c, markm,
          stplx, fnp, pg, wr1, x, &m1, wi1, wr2, wr3, wr4, wr5, s, wi2,
          mark, pnc, nexp, lastp);
ret:
   if (nd != NULL) free ((char *) nd);
   if (c != NULL) free ((char *) c);
}

static void glopt_(double *xm, const double *a, const double *b, int *m, 
   double *fm, 
   double (*f) (const double *x, int n),
   int cnmax, int pn, int fnmax, int *nd, int *reduc, double *c, 
   int *markm, double *stplx, int *fnp, int *pg, 
   double *wr1, double *x, int *m1, int *wi1, double *wr2, double *wr3, 
   double *wr4, double *wr5, double *s, int *wi2, int *mark, int *pnc, 
   int *nextp, int *lastp)
{
   int s_dim1, s_offset, x_dim1, x_offset, i__1, i__2, i__3;
   double r__1;
   double d__1, d__2;
   static double alfa;
   static int rkey;
   static double stpl;
   static int d, i, j;
   static int guess;
   static double d1;
   static int k1, k2, m2;
   static int k3;
   static double r1;
   static int istrt, mc, cn, ii, jj, sn;
   static double xk;
   static int ird, ndp, ior;

   --lastp;
   --nextp;
   --pnc;
   --mark;
   --wi2;
   s_dim1 = *m;
   s_offset = s_dim1 + 1;
   s -= s_offset;
   --wr5;
   --wr4;
   --wr3;
   --wr2;
   --wi1;
   x_dim1 = *m1;
   x_offset = x_dim1 + 1;
   x -= x_offset;
   --wr1;
   --pg;
   --fnp;
   --stplx;
   --markm;
   --c;
   --reduc;
   --nd;
   --b;
   --a;
   --xm;
   ior = NO_EXIT_MET;
   statis_1.ifail = 1;
   rkey = 1;
   ndp = 1;
   ird = 10;
   guess = 0;
   statis_1.fn = 0;
   d1 = 0.;
   i__1 = *m;
   for (i = 1; i <= i__1; ++i) {
      c[i] = b[i] - a[i];
      r__1 = c[i];
      d1 += r__1 * r__1;
   }
   d__1 = 1. /  (pn);
   d__2 = 1. /  (*m);
   stpl = sqrt(d1 /  (*m)) * pow(d__1, d__2) * .1;
   i__1 = cnmax;
   for (i = 1; i <= i__1; ++i) {
      reduc[i] = ird;
   }
   i__1 = *m;
   for (i = 1; i <= i__1; ++i) {
      nd[i] = ndp;
   }
   i__1 = pn;
   for (ii = 1; ii <= i__1; ++ii) {
      markm[ii] = 0;
      stplx[ii] = stpl;
      fnp[ii] = 1;
   }
   glopo_(m, &ndp, &pn, &guess, &a[1], &c[1], &nd[1], &pg[1], &x[x_offset], 
      &wi1[1], &wr1[1], &wr2[1], m1, f);
   statis_1.fn = statis_1.fn + pn - guess;
   i__1 = cnmax;
   for (sn = 1; sn <= i__1; ++sn) {
      clust_(&fnmax, m, &sn, &statis_1.fn, &pn, &stpl, &stplx[1], &pg[1], 
         &fnp[1], &x[x_offset], &wr1[1], &wr2[1], &wr3[1], m1, &a[1], 
         &c[1], &m2, f);
      pn = m2;
      if (! (sn == 1 || rkey == 1)) {
         goto L110;
      }
      radus_(m, m, &pn, &alfa, &x[x_offset], &pg[1], &d, &r1, &s[s_offset], 
         &wr1[1], &wr2[1], &wr3[1], &wr4[1], &wr5[1], m1);
L110:
      i__2 = pn;
      for (ii = 1; ii <= i__2; ++ii) {
         jj = pg[ii];
         wr1[ii] = x[*m1 + jj * x_dim1];
      }
      msort_(&pn, &wr1[1], &wi2[1], &istrt, &wi1[1]);
      i__2 = pn;
      for (ii = 1; ii <= i__2; ++ii) {
         wi1[ii] = pg[ii];
      }
      i__2 = pn;
      for (ii = 1; ii <= i__2; ++ii) {
         jj = pn - ii + 1;
         pg[jj] = wi1[istrt];
         istrt = wi2[istrt];
      }
      separ_(m, &pn, &d, &r1, &pg[1], &x[x_offset], &cn, &mark[1], &wi1[1], 
         &wr3[1], &wr1[1], &pnc[1], &wi2[1], &wr2[1], m1, &nextp[1], 
         &lastp[1]);
      m2 = pn;
      outre_(m, &sn, &pn, &cn, &reduc[1], &pg[1], &markm[1], &mark[1], 
         &pnc[1]);
      statis_1.it = sn;
      statis_1.itm = sn;
      ior = ispa16_(&cn, &pnc[1], &mark[1], m, &x[x_offset], fm, &xm[1],
         m1);
      if (ior == EXIT_MET) {
         goto L199;
      }
      if (pn > 1) {
         goto L145;
      }
      statis_1.ifail = 2;
      goto L199;
L145:
      if (statis_1.fn >= fnmax) {
         goto L199;
      }
   }
   statis_1.ifail = 0;
L199:
   k2 = 0;
   i__1 = cn;
   for (mc = 1; mc <= i__1; ++mc) {
      k1 = k2 + 1;
      k2 += pnc[mc];
      i__2 = k2;
      for (k3 = k1; k3 <= i__2; ++k3) {
         j = mark[k3];
         xk = x[*m1 + j * x_dim1];
         if (k3 == 1) {
            goto L200;
         }
         if (xk <= *fm) {
            goto L220;
         }
L200:
         *fm = xk;
         i__3 = *m;
         for (i = 1; i <= i__3; ++i) {
            xm[i] = x[i + j * x_dim1];
         }
L220:
         ;
      }
   }
   *fm = -(*fm);
}

static void eigval_(int *lp, double *e, double *a, double *b, double *w, 
   double *f)
{
   int i__1, i__2;
   double r__1, r__2, r__3;
   double d__1;
   static int i, j, k, n;
   static double x, s1, s2;
   static double bd, am, bm;

   --f;
   --w;
   --b;
   --a;
   --e;
   am = fabs(a[1]);
   bm = 0.;
   i__1 = *lp;
   for (i = 2; i <= i__1; ++i) {
      r__2 = am, r__3 = (r__1 = a[i], fabs(r__1));
      am =  MAX(r__2,r__3);
      r__2 = bm, r__3 = (r__1 = b[i], fabs(r__1));
      bm =  MAX(r__2,r__3);
   }
   bd = am + bm + bm;
   i__1 = *lp;
   for (i = 1; i <= i__1; ++i) {
      a[i] /= bd;
      b[i] /= bd;
      e[i] = -1.;
      w[i] = 1.;
   }
   i__1 = *lp;
   for (k = 1; k <= i__1; ++k) {
L8:
      d__1 = (r__1 = w[k], fabs(r__1)) + (r__2 = e[k], fabs(r__2));
      if ((w[k] - e[k]) / MAX(d__1,1e-10) - r1mach (4) * 1e3 
         <= 0.) {
         goto L50;
      } else {
         goto L10;
      }
L10:
      x = (w[k] + e[k]) * .5;
      s2 = 1.;
      f[1] = a[1] - x;
      if (f[1] >= 0.) {
         goto L104;
      } else {
         goto L102;
      }
L102:
      s1 = -1.;
      n = 0;
      goto L105;
L104:
      s1 = 1.;
      n = 1;
L105:
      i__2 = *lp;
      for (i = 2; i <= i__2; ++i) {
         if ((r__1 = b[i], fabs(r__1)) <= r1mach

 (1)) {
            goto L113;
         }
         if ((r__1 = b[i - 1], fabs(r__1)) <= r1mach (1)) {
            goto L114;
         }
         if ((r__1 = f[i - 1], fabs(r__1)) + (r__2 = f[i - 2], fabs(r__2)) 
            - 1e-10 >= 0.) {
            goto L112;
         } else {
            goto L111;
         }
L111:
         f[i - 1] *= 1e10;
         f[i - 2] *= 1e10;
L112:
         f[i] = (a[i] - x) * f[i - 1] - b[i] * b[i] * f[i - 2];
         goto L115;
L113:
         f[i] = (a[i] - x) * s1;
         goto L115;
L114:
         r__1 = b[i] * b[i];
         f[i] = (a[i] - x) * f[i - 1] - sign(r__1, s2);
L115:
         s2 = s1;
         if ((r__1 = f[i], fabs(r__1)) <= r1mach (1)) {
            goto L117;
         }
         s1 = sign(s1, f[i]);
         if ((r__1 = s1 + s2, fabs(r__1)) <= r1mach (1)) {
            goto L120;
         }
L117:
         ++n;
L120:
         ;
      }
      n = *lp - n;
      if (n < k) {
         goto L20;
      }
      i__2 = n;
      for (j = k; j <= i__2; ++j) {
         w[j] = x;
      }
L20:
      ++n;
      if (*lp < n) {
         goto L8;
      }
      i__2 = *lp;
      for (j = n; j <= i__2; ++j) {
         if (x - e[j] <= 0.) {
            goto L8;
         } else {
            goto L26;
         }
L26:
         e[j] = x;
      }
      goto L8;
L50:
      ;
   }
   i__1 = *lp;
   for (i = 1; i <= i__1; ++i) {
      a[i] *= bd;
      b[i] *= bd;
      w[i] = (w[i] + e[i]) * bd * .5;
   }
   j = *lp;
   k = 1;
   i__1 = *lp;
   for (i = 1; i <= i__1; ++i) {
      if ((r__1 = w[k], fabs(r__1)) - (r__2 = w[j], fabs(r__2)) <= 0.) {
         goto L63;
      } else {
         goto L65;
      }
L63:
      e[i] = w[j];
      --j;
      goto L80;
L65:
      e[i] = w[k];
      ++k;
L80:
      ;
   }
}

static void tridmx_(int *n, int *nm, double *a, double *d, double *b)
{
   int a_dim1, a_offset, i__1, i__2, i__3;
   double r__1, r__2;
   static double scal;
   static int i, j, k, l;
   static double t, denom;
   extern double r1mach(int i);
   static int jj, kk;
   static double sum;

   --b;
   --d;
   a_dim1 = *nm;
   a_offset = a_dim1 + 1;
   a -= a_offset;
   i__1 = *n;
   for (i = 1; i <= i__1; ++i) {
      d[i] = a[i + i * a_dim1];
   }
   if ((i__1 = *n - 2) < 0) {
      goto L60;
   } else if (i__1 == 0) {
      goto L55;
   } else {
      goto L15;
   }
L15:
   i__1 = *n;
   for (k = 3; k <= i__1; ++k) {
      kk = k - 1;
      sum = a[k - 1 + (k - 2) * a_dim1] * a[k - 1 + (k - 2) * a_dim1];
      i__2 = *n;
      for (j = k; j <= i__2; ++j) {
         sum += a[j + (k - 2) * a_dim1] * a[j + (k - 2) * a_dim1];
      }
      r__1 = sqrt(sum);
      r__2 = -a[k - 1 + (k - 2) * a_dim1];
      b[k - 2] = sign(r__1, r__2);
      if ((r__1 = b[k - 2], fabs(r__1)) <= r1mach (1)) {
         goto L46;
      }
      a[k - 1 + (k - 2) * a_dim1] = sqrt((r__1 = a[k - 1 + (k - 2) * a_dim1]
         / b[k - 2], fabs(r__1)) * .5 + .5);
      denom = a[k - 1 + (k - 2) * a_dim1] * -2. * b[k - 2];
      i__2 = *n;
      for (i = k; i <= i__2; ++i) {
         a[i + (k - 2) * a_dim1] /= denom;
      }
      scal = 0.;
      i__2 = *n;
      for (j = kk; j <= i__2; ++j) {
         b[j] = 0.;
         if (j == kk) {
           goto L350;
         }
         i__3 = jj;
         for (l = kk; l <= i__3; ++l) {
            b[j] += a[j + l * a_dim1] * a[l + (k - 2) * a_dim1];
         }
L350:
         i__3 = *n;
         for (l = j; l <= i__3; ++l) {
            b[j] += a[l + j * a_dim1] * a[l + (k - 2) * a_dim1];
         }
         jj = j;
         scal += b[j] * a[j + (k - 2) * a_dim1];
      }
      i__2 = *n;
      for (j = kk; j <= i__2; ++j) {
         b[j] -= scal * a[j + (k - 2) * a_dim1];
      }
      i__2 = *n;
      for (j = kk; j <= i__2; ++j) {
         i__3 = *n;
         for (l = j; l <= i__3; ++l) {
            a[l + j * a_dim1] -= (a[l + (k - 2) * a_dim1] * b[j] + a[j + (
               k - 2) * a_dim1] * b[l]) * 2.;
         }
      }
L46:
      ;
   }
   i__1 = *n;
   for (i = 1; i <= i__1; ++i) {
      t = a[i + i * a_dim1];
      a[i + i * a_dim1] = d[i];
      j = *n - i;
      b[j + 1] = b[j];
      d[i] = t;
   }
L55:
   b[*n] = a[*n + (*n - 1) * a_dim1];
L60:
   b[1] = 0.;
}

static void glopo_(int *m, int *ndp, int *n, int *guess, 
   const double *a, double *c, int *nd, int *pg, double *x, int *step, 
   double *r, double *z, int *m1, double (*f) (const double *x, int n))
{
   int x_dim1, x_offset, i__1;
   static int i, j, k;
   static double ri;

   --z;
   --r;
   --step;
   x_dim1 = *m1;
   x_offset = x_dim1 + 1;
   x -= x_offset;
   --pg;
   --nd;
   --c;
   --a;
   k = *guess;
   switch (*ndp) {
      case 1:  goto L10;
      case 2:  goto L7;
   }
L4:
   ++k;
   x[*m + 1 + k * x_dim1] = -(*f)(&r[1], *m);
   i__1 = *m;
   for (i = 1; i <= i__1; ++i) {
      x[i + k * x_dim1] = r[i];
   }
   pg[k] = k;
   if (k < *n) {
      switch (*ndp) {
         case 1:  goto L10;
         case 2:  goto L30;
      }
   }
   return;
L7:
   i__1 = *m;
   for (i = 1; i <= i__1; ++i) {
      step[i] = 1;
   }
L10:
   i__1 = *m;
   for (i = 1; i <= i__1; ++i) {
      undev_(&ri);
      r[i] = a[i] + c[i] * ri;
      z[i] = r[i];
   }
L20:
   j = *m;
   if (ira_(&r[1], &a[1], &c[1], m) == 1) {
      goto L4;
   }
   switch (*ndp) {
      case 1:  goto L10;
      case 2:  goto L30;
   }
L30:
   ++step[j];
   if (step[j] > nd[j]) {
      goto L40;
   }
   r[j] += c[j];
   goto L20;
L40:
   step[j] = 1;
   r[j] = z[j];
   --j;
   if (j <= 0) {
      goto L10;
   } else {
      goto L30;
   }
}

static void clust_(int *fenmx, int *m, int *cc, int *fen, int *n, double *stpl,
   double *stplx, int *pg, int *fenp, 
   double *x, double *y, double *z, double *s, int *m1, const double *a, 
   double *c, 
   int *m2, double (*f) (const double *x, int n))
{
   int x_dim1, x_offset, i__1, i__2;
   static int i, j, k, l;
   static double sf, sl, ss;
   static int new_;

   --c;
   --a;
   --s;
   --z;
   --y;
   x_dim1 = *m1;
   x_offset = x_dim1 + 1;
   x -= x_offset;
   --fenp;
   --pg;
   --stplx;
   *m2 = *n;
   sl = *stpl;
   *stpl *= .3;
   if (*stpl < r1mach (1)) {
      *stpl = r1mach (1);
   }
   i__1 = *n;
   for (j = 1; j <= i__1; ++j) {
      k = pg[j];
      if (*fen >= *fenmx) {
         goto L90;
      }
      sf = stplx[k];
      i__2 = *m1;
      for (i = 1; i <= i__2; ++i) {
         y[i] = x[i + k * x_dim1];
      }
      l = *fen;
      locop_(m, &y[1], &sf, &ss, &sl, &new_, fen, fenmx, &z[1], &s[1], m1, 
         &a[1], &c[1], f);
      fenp[k] = fenp[k] + *fen - l;
      stplx[k] = ss;
      if (new_ < 0) {
         goto L90;
      } else if (new_ == 0) {
         goto L100;
      } else {
         goto L17;
      }
L17:
      i__2 = *m1;
      for (i = 1; i <= i__2; ++i) {
         x[i + k * x_dim1] = y[i];
      }
      goto L100;
L90:
      if (*cc == 1) {
         goto L110;
      }
L100:
      ;
   }
   return;
L110:
   *m2 = j;
}

static void locop_(int *m, double *y, double *sf, double *sl, double *sll, 
   int *new_, int *fen, int *fenmx, double *x, double *s, 
   int *m1, const double *a, double *c,
   double (*f) (const double *x, int n))
{
   int i__1, i__2, i__3;
   static int half, i, j, k, l;
   static double q;
   static int dub;

   --c;
   --a;
   --s;
   --x;
   --y;
   *new_ = 0;
   *sl = *sf;
   half = 2;
   dub = 1;
L20:
   i__1 = half;
   for (j = 1; j <= i__1; ++j) {
      if (*fen == *fenmx) {
         return;
      }
      l = 1;
      q = 0.;
      i__2 = *m;
      for (i = 1; i <= i__2; ++i) {
         normd_(&s[i]);
         q += s[i] * s[i];
      }
      q = *sl / sqrt(q);
L60:
      i__2 = *m;
      for (i = 1; i <= i__2; ++i) {
         s[i] = q * s[i];
      }
L70:
      i__2 = *m;
      for (i = 1; i <= i__2; ++i) {
         x[i] = y[i] + s[i];
      }
      i__2 = dub;
      for (k = 1; k <= i__2; ++k) {
         if (*fen < *fenmx) {
            goto L90;
         }
         *new_ = -1 - *new_;
         return;
L90:
         if (ira_(&x[1], &a[1], &c[1], m) == 0) {
            switch (l) {
            case 1:  goto L160;
            case 2:  goto L200;
            }
         }
         x[*m1] = -(*f) (&x[1], *m);
         ++(*fen);
         if (x[*m1] <= y[*m1]) {
            switch (l) {
            case 1:  goto L160;
            case 2:  goto L200;
            }
         }
         i__3 = *m;
         for (i = 1; i <= i__3; ++i) {
            y[i] = x[i];
            x[i] += s[i];
         }
         y[*m1] = x[*m1];
         ++(*new_);
      }
      *sl *= 2.;
      q = 2.;
      goto L60;
L160:
      l = 2;
      i__2 = *m;
      for (i = 1; i <= i__2; ++i) {
         s[i] = -s[i];
      }
      goto L70;
L200:
      ;
   }
   *sl *= .5;
   if (*sl > *sll) {
      goto L20;
   }
}

static void radus_(int *dim, int *dimax, int *n, double *alfa, 
   double *x, int *p, int *d, double *r1, double *s, double *a, 
   double *b, double *e, double *w, double *f, int *m1)
{
   int x_dim1, x_offset, s_dim1, s_offset, i__1, i__2, i__3;
   double r__1, r__2;
   double d__1, d__2;
   static double dinv, ninv, sqin;
   static int i, j, k, m;
   static double toalf;
   static int i1, i2;
   static double eh;
   static int jj;
   static double xh;
   static double f2d, f1m;
   static double r1n;
   static short odd;
   static double phi;

   --f;
   --w;
   --e;
   --b;
   --a;
   s_dim1 = *dimax;
   s_offset = s_dim1 + 1;
   s -= s_offset;
   --p;
   x_dim1 = *m1;
   x_offset = x_dim1 + 1;
   x -= x_offset;
   ninv = 1. /  (*n);
   sqin = sqrt(ninv);
   phi = 3.141593;
   *alfa = 2.;
   i__1 = *dim;
   for (i = 1; i <= i__1; ++i) {
      xh = 0.;
      i__2 = *n;
      for (jj = 1; jj <= i__2; ++jj) {
         j = p[jj];
         xh += x[i + j * x_dim1];
      }
      w[i] = xh * sqin;
   }
   i__1 = *dim;
   for (i1 = 1; i1 <= i__1; ++i1) {
      i__2 = i1;
      for (i2 = 1; i2 <= i__2; ++i2) {
         xh = 0.;
         i__3 = *n;
         for (jj = 1; jj <= i__3; ++jj) {
            j = p[jj];
            xh += x[i1 + j * x_dim1] * x[i2 + j * x_dim1];
         }
         xh -= w[i1] * w[i2];
         s[i1 + i2 * s_dim1] = xh;
         s[i2 + i1 * s_dim1] = xh;
      }
   }
   r__1 = ninv, r__2 = r__1;
   *r1 = *alfa * *alfa * s[s_dim1 + 1] * (r__2 * (r__1 * r__1));
   if (*dim == 1) {
      goto L39;
   }
   tridmx_(dim, dimax, &s[s_offset], &a[1], &b[1]);
   eigval_(dim, &e[1], &a[1], &b[1], &w[1], &f[1]);
   *r1 = -1.;
   f2d = 1.;
   f1m = 1.;
   odd = TRUE;
   m = 0;
   eh = 1.;
   toalf = *alfa * 4. * *alfa * ninv;
   i__1 = *dim;
   for (k = 1; k <= i__1; ++k) {
      if (e[k] < 0.) {
         e[k] = 0.;
      }
      eh = eh * e[k] * toalf;
      e[k] = sqrt(eh);
      dinv = 2. /  k;
      if (odd) {
         goto L32;
      }
      d__1 =  (f1m * e[k] * ninv);
      d__2 =  dinv;
      r1n = pow(d__1, d__2) / phi;
      odd = TRUE;
      f2d *=  (k + 1);
      goto L34;
L32:
      r__1 = phi * 2.;
      d__1 =  (f2d * e[k] * ninv / (pow(r__1, (double)m) * 
        2.));
      d__2 =  dinv;
      r1n = pow(d__1, d__2);
      ++m;
      f1m *=  m;
      odd = FALSE;
L34:
      if (*r1 >= r1n) {
         goto L40;
      }
      *r1 = r1n;
   }
L39:
   *d = *dim;
   return;
L40:
   *d = k - 1;
}

static void msort_(int *n, double *a, int *list, int *start, int *hds)
{
   static int try2;
   static double next;
   static int j, k, t, it;
   static double at1, at2;
   static int nol;
   static double try_;
   static int try1;

   --hds;
   --list;
   --a;
   *start = 1;
   list[1] = 0;
   if (*n <= 1) {
      return;
   }
   j = 1;
   t = 1;
   k = 2;
   try_ = a[1];
L10:
   next = a[k];
   if (try_ > next) {
      goto L110;
   }
   hds[j] = t;
L50:
   list[t] = k;
   if (k - *n != 0) {
      goto L70;
   } else {
      goto L60;
   }
L60:
   list[k] = 0;
   goto L150;
L70:
   try_ = next;
   t = k;
   ++k;
   next = a[k];
   if (try_ <= next) {
      goto L50;
   }
   list[t] = 0;
L80:
   t = k;
   ++k;
   ++j;
   if (t - *n != 0) {
      goto L100;
   } else {
      goto L90;
   }
L90:
   hds[j] = *n;
   list[*n] = 0;
   goto L150;
L100:
   try_ = next;
   goto L10;
L110:
   list[t] = 0;
L120:
   list[k] = t;
   if (k - *n != 0) {
      goto L140;
   } else {
      goto L130;
   }
L130:
   hds[j] = k;
   goto L150;
L140:
   try_ = next;
   t = k;
   ++k;
   next = a[k];
   if (try_ >= next) {
      goto L120;
   }
   hds[j] = t;
   goto L80;
L150:
   if (j == 1) {
      goto L320;
   }
   nol = j;
   it = 1;
L155:
   if (it >= nol) {
      goto L160;
   }
   k = it;
   it += it;
   goto L155;
L160:
   j = k + k + 1 - nol;
   t = j;
   goto L180;
L170:
   t = 1;
   j = 1;
L180:
   try1 = hds[t];
   try2 = hds[t + 1];
   at1 = a[try1];
   at2 = a[try2];
   if (at1 - at2 <= 0.) {
      goto L190;
   } else {
      goto L200;
   }
L190:
   hds[j] = try1;
   goto L260;
L200:
   hds[j] = try2;
L210:
   k = list[try2];
   if (k != 0) {
      goto L230;
   } else {
      goto L220;
   }
L220:
   list[try2] = try1;
   goto L310;
L230:
   at2 = a[k];
   if (at1 - at2 >= 0.) {
      goto L240;
   } else {
      goto L250;
   }
L240:
   try2 = k;
   goto L210;
L250:
   list[try2] = try1;
   try2 = k;
L260:
   k = list[try1];
   if (k != 0) {
      goto L280;
   } else {
      goto L270;
   }
L270:
   list[try1] = try2;
   goto L310;
L280:
   at1 = a[k];
   if (at2 - at1 >= 0.) {
      goto L300;
   } else {
      goto L290;
   }
L290:
   list[try1] = try2;
   try1 = k;
   goto L210;
L300:
   try1 = k;
   goto L260;
L310:
   ++j;
   t += 2;
   if (nol > t) {
      goto L180;
   }
   nol = j - 1;
   if (nol > 1) {
      goto L170;
   }
L320:
   *start = hds[1];
}

static void separ_(int *dim, int *n, int *d, double *r1, 
   int *p, double *x, int *ncl, int *mark, int *markr, 
   double *c, double *r, int *nmcv, int *distp, double *dist, int *m1, 
   int *nextp, int *lastp)
{
   int x_dim1, x_offset, i__1, i__2, i__3, i__4;
   double d__1, d__2;
   static int left;
   static double dinv, e;
   static int i, j, k, ilast, nlast;
   static double d2;
   static int nnext, ic, ii, jj, kk, kl, jl, jp, is, ix;
   static double rr;
   static int kk1, kk2, is1, nmc, nmt, nrr;

   --lastp;
   --nextp;
   --dist;
   --distp;
   --nmcv;
   --r;
   --c;
   --markr;
   --mark;
   x_dim1 = *m1;
   x_offset = x_dim1 + 1;
   x -= x_offset;
   --p;
   i__1 = *n;
   for (i = 1; i <= i__1; ++i) {
      markr[i] = 0;
   }
   *ncl = 0;
   nmt = 0;
   left = *n;
   is = 1;
   dinv = 2. /  (*d);
   r[1] = *r1;
   nrr = 1;
L20:
   ++(*ncl);
   nmc = 1;
   ++nmt;
   --left;
   k = p[is];
   mark[nmt] = k;
   markr[is] = *ncl;
   if (left == 0) {
      goto L60;
   }
   i__1 = *dim;
   for (i = 1; i <= i__1; ++i) {
      c[i] = x[i + k * x_dim1];
   }
   kl = is;
   i__1 = left;
   for (kk = 1; kk <= i__1; ++kk) {
L44:
      ++kl;
      if (markr[kl] != 0) {
         goto L44;
      }
      d2 = 0.;
      ix = p[kl];
      i__2 = *dim;
      for (i = 1; i <= i__2; ++i) {
         e = c[i] - x[i + ix * x_dim1];
         d2 += e * e;
      }
      dist[kk] = d2;
      distp[kk] = kl;
   }
   kk2 = left;
   i__1 = kk2;
   for (i = 1; i <= i__1; ++i) {
      kk1 = left;
      if (i <= nrr) {
         goto L505;
      }
      d__1 =  i;
      d__2 =  dinv;
      r[i] = *r1 * pow(d__1, d__2);
      ++nrr;
L505:
      rr = r[i];
      if (i == 1) {
         goto L510;
      }
      i__2 = nnext;
      for (ii = 1; ii <= i__2; ++ii) {
         lastp[ii] = nextp[ii];
      }
      nlast = nnext;
L510:
      nnext = 0;
      j = 0;
      i__2 = kk1;
      for (jl = 1; jl <= i__2; ++jl) {
L512:
         ++j;
         if (distp[j] == 0) {
            goto L512;
         }
         if (dist[j] > rr) {
            goto L56;
         }
         ic = distp[j];
         ic = p[ic];
         if (i == 1) {
            goto L548;
         }
         i__3 = *dim;
         for (ii = 1; ii <= i__3; ++ii) {
            c[ii] = x[ii + ic * x_dim1];
         }
         i__3 = nlast;
         for (jj = 1; jj <= i__3; ++jj) {
            d2 = 0.;
            ilast = lastp[jj];
            i__4 = *dim;
            for (ii = 1; ii <= i__4; ++ii) {
               e = c[ii] - x[ii + ilast * x_dim1];
               d2 += e * e;
            }
            if (d2 <= *r1) {
               goto L548;
            }
         }
         goto L56;
L548:
         ++nnext;
         nextp[nnext] = ic;
         ++nmc;
         ++nmt;
         --left;
         jp = distp[j];
         mark[nmt] = p[jp];
         markr[jp] = *ncl;
         distp[j] = 0;
         if (left == 0) {
            goto L60;
         }
L56:
         ;
      }
      if (left == kk1) {
         goto L60;
      }
   }
L60:
   nmcv[*ncl] = nmc;
   if (left == 0) {
      return;
   }
   is1 = is + 1;
   i__1 = *n;
   for (is = is1; is <= i__1; ++is) {
      if (markr[is] == 0) {
         goto L20;
      }
   }
}

static void outre_(int *m, int *sn, int *pn, int *cn, int *reduc, int *pg, 
   int *markm, int *mark, int *pnc)
{
   int i__1, i__2, i__3;
   static int j, k, m1, mc, jf, jl, js;

   --pnc;
   --mark;
   --markm;
   --pg;
   --reduc;
   *pn = 0;
   js = reduc[*sn];
   m1 = *m + 1;
   jl = 0;
   i__1 = *cn;
   for (mc = 1; mc <= i__1; ++mc) {
      jf = jl + 1;
      jl += pnc[mc];
      i__2 = jl;
      for (j = jf; j <= i__2; ++j) {
         k = mark[j];
         markm[k] = mc;
      }
      i__2 = jl;
      i__3 = js;
      for (j = jf; i__3 < 0 ? j >= i__2 : j <= i__2; j += i__3) {
         ++(*pn);
         pg[*pn] = mark[j];
      }
   }
}

static void undev_(double *a)
{
   *a = ats();
}

static void normd_(double *a)
{
   *a = sin(ats() * 6.283185) * sqrt(log(ats()) * -2.);
}

static int ira_(double *x, const double *a, double *c, int *m)
{
   int ret_val, i__1;
   static int i;
   static double x1;
   static int ir;

   --c;
   --a;
   --x;
   ir = 1;
   i__1 = *m;
   for (i = 1; i <= i__1; ++i) {
      x1 = x[i] - a[i];
      if (x1 < 0. || x1 > c[i]) {
         ir = 0;
      }
   }
   ret_val = ir;
   return ret_val;
}

static int ispa16_(int *nc, int *pnc, int *mark, int *n, double *x,
   double *fm, double *xm, int *m1sp)
{
   int x_dim1, x_offset, ret_val, i__1, i__2, i__3;
   static int i, j, k;
   static double f1;
   static int m1;
   static int mc, jf, jl;

   --xm;
   x_dim1 = *m1sp;
   x_offset = x_dim1 + 1;
   x -= x_offset;
   --mark;
   --pnc;
   m1 = *n + 1;
   jl = 0;
   i__1 = *nc;
   for (mc = 1; mc <= i__1; ++mc) {
      jf = jl + 1;
      jl += pnc[mc];
      i__2 = jl;
      for (j = jf; j <= i__2; ++j) {
         k = mark[j];
         f1 = x[m1 + k * x_dim1];
         if (mc == 1 && j == jf) {
            goto L50;
         }
         if (f1 <= *fm) {
            goto L20;
         }
L50:
         *fm = f1;
         i__3 = *n;
         for (i = 1; i <= i__3; ++i) {
            xm[i] = x[i + k * x_dim1];
         }
L20:
         ;
      }
   }
   f1 = -(*fm);
   fafm [statis_2 .nr - 1] = f1;
   copy_array (faxn [statis_2 .nr - 1], xm + 1, *n);
   ret_val = met_out (statis_2.nr);
   return ret_val;
}

