/* File: lbayes.C */

#include <stdio.h>
#include <malloc.h>
#include <values.h>
#include <stdlib.h>
#include <math.h>
#include "const.h"
#include "res.h"
#include "task.h"
#include "ERROR.h"
#include "OUTPUT.h"

#define EPS0           0.5
#define A0             1.0
#define GAMA0          0.2
#define COEF           0.8
#define EPS2           1e-6
#define EPS3           1e-6
#define EPS4           1e-10
#define B1             1e-6
#define V1             1e-6
#define B2             1e-2
#define V2             1.0
#define CONV           1
#define SL_CONV        2
#define M1             (COEF * max_it - 2)

static void lbayes0 (double *x, const double *a, const double *b, int n,
   double *y, double (*f) (const double *x, int n), int max_it, double aniu, 
   double beta);
static void lbayes_dispersion (double (*f) (const double *x, int n), 
   const double *x, double *x1, int n,
   double *mean, double *disp, int n2);

static void lbayes_dispersion (double (*f) (const double *x, int n), 
   const double *x, double *x1, int n,
   double *mean, double *disp, int n2)
{
   int i;

   for (i = 0; i < n2; i++)
      x1 [i] = (*f) (x, n);
   dispersion (x1, n2, mean, disp);
}

/*---------------------------------------------------------------------------*/

#define N2  (2 * n)

static double *fm;
static double **xn;
static double *x1 = NULL;
static double *x2, *xbest, *dml, *dml1, *ab;

void lbayes (double *x, const double *a, const double *b, int n, double *y,
   double (*f) (const double *x, int n), int max_it, double aniu, double beta,
   double *pfm, double **pxn)
{
   fm = pfm;
   xn = pxn;
   if ((x1 = (double *) malloc (sizeof (double) *
      (5 * n + N2))) == NULL) {
      ERROR ("out of memory", False);
      goto ret;
   }
   x2    = x1 + n;
   dml   = x2 + N2;
   dml1  = dml + n;
   ab    = dml1 + n;
   xbest = ab + n;
   lbayes0 (x, a, b, n, y, f, max_it, aniu, beta);
ret:
   if (x1 != NULL) {
      free ((char *) x1);
      x1 = NULL;
   }
}

static void lbayes0 (double *x, const double *a, const double *b, int n,
   double *y, double (*f) (const double *x, int n), int max_it, double aniu, 
   double beta)
{
   double ff, f11, f13;
   int    i, m;
   int    n2    = N2;
   int    am2   = 1;
   int    an0   = 1;
   int    lo    = CONV;
   int    lb    = 1;
   double a1    = 0.;

   copy_array (xbest, x, n);
   lbayes_dispersion (f, x, x2, n, y, &f11, n2);

   int n_of_points = 0;

   fm [n_of_points] = ff = *y;
   copy_array (xn [n_of_points], x, n);
   n_of_points++;
   if (met_out (n_of_points) == EXIT_MET) goto end;
   f13 = n2 * (*y);
   for (i = 0; i < n; i++) {
      ab [i] = 0.5 * (b [i] - a [i]);
   }
   for (m = 0; m < max_it - 1; m++) {
      double hm0, hm, s1, am, d1, epsm, sig0, a3, f8, f12, f14;

      hm0       = GAMA0 * pow ((double) (m + 1), -aniu);
      copy_array (x1, x, n);
      copy_array (x2, x, n);
      for (i = 0; i < n; i++) {
         double dx;

         dx       = hm0 * ab [i];
         x1 [i]   = increment (x [i], a [i], b [i],  dx);
         x2 [i]   = increment (x [i], a [i], b [i], -dx);
         dml [i]  = 2 * dx * ((*f) (x1, n) - (*f) (x2, n));
         dml [i] /= x1 [i] - x2 [i];
         x1 [i]   = x2 [i] = x [i];
      }
      if (lb > 1) {
         for (i = 0; i < n; i++) {
            dml [i] = (dml [i] + dml1 [i] * (lb - 1)) / lb;
         }
      }
      s1 = norma (dml ,n) + EPS4;
      hm = sqrt (hm0) / s1;
      for (i = 0; i < n; i++) {
         double dx;

         dx      = hm * ab [i] * dml [i];
         x1 [i]  = increment (x [i], a [i], b [i],  dx);
         x2 [i]  = increment (x [i], a [i], b [i], -dx);
      }
      {
         double hs  = 0.;
         double hs1 = 0.;
         double hs2 = 0.;

         for (i = 0; i < n; i++) {
            hs  += POW_2 ((x2 [i] - x1 [i]) / ab [i]);
            hs1 += POW_2 ((x1 [i] - x  [i]) / ab [i]) + EPS3;
            hs2 += POW_2 ((x2 [i] - x  [i]) / ab [i]) + EPS3;
         }
         hs  = sqrt (hs);
         hs1 = sqrt (hs1);
         hs2 = sqrt (hs2);
         am  = (*f) (x1, n) * hs2 + (*f) (x2, n) * hs1 - (*y) * hs;
         am *= 2 * hm0;
         am /= hs1 * hs2 * (hs1 + hs2);
      }
      d1 = am;
      if (am < B1) {
         d1 = V1;
      }
      if ((am < B2) && (m == 0)) {
         d1 = V2;
      }
      a1  += d1 / hm0;
      epsm = hm0 * a1 / (m + 1);
      if (m < M1) {
         double sigl;

         if (f11 * (n2 - 1) / n2 < EPS2) {
            f11 = 2 * EPS2;
            n2 = 2;
         }
         sig0 = sqrt (f11);
         sigl = hm0 * sig0 * sqrt (n / (2. * lb)) / d1;
         if (lo != CONV) {
            if (s1 >= sigl) {
               lo = CONV;
            } else {
               an0++;
            }
         } else {
            if (s1 <= 0.5 * sigl) {
               lo = SL_CONV;
               an0++;
            }
         }
      } else {
         an0++;
      }
      a3 = (pow ((double) (an0), beta) + 2) * epsm;
      for (i = 0; i < n; i++)
         x1 [i] = increment (x [i], a [i], b [i], -ab [i] * dml [i] / a3);
      lbayes_dispersion (f, x1, x2, n, &f8, &f12, n2);
      f14  = n2 * f8;
      if (f8 <= *y + 0.5 * sig0 / sqrt((double) (n2))) {
         copy_array (x, x1, n);
         *y  = f8;
         f13 = f14;
         am2++;
         f11 = (f12 + f11 * (am2 - 1)) / am2;
         lb  = 1;
      } else {
         f13 += (*f) (x, n);
         if (am2 == 1) {
            *y = f13 / (lb + 2 * n);
         } else {
            *y = f13 / (lb + n2);
         }
         lb++;
         copy_array (dml1, dml, n);
      }
      if (*y < ff) {
         ff = *y;
         copy_array (xbest, x, n);
      }
      fm [n_of_points] = *y;      
      copy_array (xn [n_of_points], x, n);
      n_of_points++;
      if (met_out (n_of_points) == EXIT_MET) goto end;
   }
end:
   copy_array (x, xbest, n);
   *y = ff;
}
